import importlib_metadata
import tomlkit
from beautifultable import BeautifulTable


def load_toml(filename):
    """
    Load toml file.

    :param filename: filename
    :return: toml body
    :rtype: dict

    """
    with open(filename) as f:
        return tomlkit.parse(f.read())


def get_package_names(pyproject):
    """
    Get package names

    :param dict pyproject: pyproject.toml body.
    :return: Package names
    :rtype: list

    """
    package_names = []
    for pkg in pyproject["package"]:
        if pkg["category"] == "main":
            package_names.append(pkg["name"])
    return package_names


def get_packages_metadata(package_names):
    """
    Get list of installed package's metadata

    :param list package_names: Install package names.
    :return: list of dict. Dict contains package name and license.
    :rtype: list

    """
    results = []
    for name in package_names:
        d = {"name": name}
        try:
            metadata = importlib_metadata.metadata(name)
        except importlib_metadata.api.PackageNotFoundError:
            continue
        else:
            d["version"] = metadata["Version"]
            d["license"] = metadata["License"]
            results.append(d)
    return results


def create_table(rows, order, style):
    """
    Create table for display

    :param list rows: table rows
    :param str order: table sorted order
    :param str style: table style
    :return: table for display
    :rtype: BeautifulTable

    """
    table = BeautifulTable()
    table.column_headers = ["name", "license"]
    table.column_alignments["name"] = BeautifulTable.ALIGN_LEFT
    table.column_alignments["license"] = BeautifulTable.ALIGN_LEFT

    if style == "markdown":
        table.set_style(BeautifulTable.STYLE_MARKDOWN)
    elif style == "rst":
        table.set_style(BeautifulTable.STYLE_RESTRUCTURED_TEXT)
    else:
        table.set_style(BeautifulTable.STYLE_COMPACT)

    for row in rows:
        table.append_row(row)

    if order == "license":
        table.sort("license")

    return table



import sys

from cleo import Command

from .core import create_table
from .core import get_package_names
from .core import get_packages_metadata
from .core import load_toml


class ListCommand(Command):
    """
    List the licenses for external packages used in your repository.

    list
        {--o|order=name : Control display order by "name" or "license".}
        {--f|format=compact : Select the output format among: "compact", "markdown", or "rst"}

    """

    def handle(self):
        try:
            pyproject = load_toml("poetry.lock")
        except FileNotFoundError:
            self.line_error("ERROR: The lock file not found.", style="error")
            sys.exit(1)

        package_names = get_package_names(pyproject)
        if not package_names:
            self.line("No data")
            sys.exit(1)

        results = get_packages_metadata(package_names)

        rows = [(p["name"], p["license"]) for p in results]

        order = self.option("order")
        style = self.option("format")
        table = create_table(rows, order=order, style=style)

        print(table)



#!/usr/bin/env python
from cleo import Application

from . import __version__
from .commands import ListCommand

application = Application("licencia", __version__, complete=True)
application.add(ListCommand())



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['licencia']

package_data = \
{'': ['*']}

install_requires = \
['beautifultable>=0.6.0,<0.7.0',
 'cleo>=0.7.2,<0.8.0',
 'importlib-metadata>=0.8.0,<0.9.0',
 'tomlkit>=0.5.3,<0.6.0']

entry_points = \
{'console_scripts': ['licencia = licencia.app:application.run']}

setup_kwargs = {
    'name': 'licencia',
    'version': '0.1.1',
    'description': '',
    'long_description': None,
    'author': 'kk6',
    'author_email': 'hiro.ashiya@gmail.com',
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
